/*
	Access serial port.
	vsantos, 16 Jun 2005 
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <termio.h>


#define USERBAUDRATE B9600
/*#define DEVICENAME "/dev/ttyS0"*/
#define DEVICENAME "/dev/ttya"

struct termio io_parm = 
{
	IGNBRK /*+IXON+IXOFF*/,
	0,
	USERBAUDRATE+CS8/*+CREAD+HUPCL*/,
	0
};



#define MSG_ORD	0x40   /*Message type to impose settings*/
#define MSG_REQ	0x80   /*Message type to request for data*/

#define PE_DIR	 0
#define PE_ESQ	 (1<<2)
#define ANCA_DIR (2<<2)
#define ANCA_ESQ (3<<2)
#define TRONCO	 (4<<2)
#define BRA_DIR  (5<<2)
#define BRA_ESQ  (5<<2)
#define CABECA	 (6<<2)

#define LADO	0
#define FRENTE	1
#define JOELHO	2

#define VERTI	0   /*Seria vantajoso este ter o numero 2 */
#define ABRIR	1   /*Seria vantajoso chamar a isto LADO e dar-lhe o valor 0*/
#define AVANTE	2   /*Seria vantajoso chamar a isto FRENTE e dar-lhe o valor 1*/

#define TOT_JOINTS 12

typedef struct { char jointPos, jointVel;} jPs;
typedef struct { char jointNum, *jointName ; } jID ;
typedef struct { jID jointID; jPs jointParams ; } jointData;

jointData HumanoidJointSequence[TOT_JOINTS]= {
///////* NOTA ESTES VALORES DE DEFEITO TEM DE SER CORRIGIDOS*******/
	 { { PE_DIR | LADO ,    "pe direito lado   "} , { 90 , 3 } }
	,{ { PE_DIR | FRENTE ,  "pe direito frente "} , { 90, 3 } }
	,{ { PE_DIR | JOELHO ,  "pe direito joelho "} , { 90, 3 } }

	,{ { PE_ESQ | LADO ,    "pe esquerdo lado  "} , { 90, 3 } }
	,{ { PE_ESQ | FRENTE ,  "pe esquerdo frente"} , { 90, 3 } }
	,{ { PE_ESQ | JOELHO ,  "pe esquerdo joelho"} , { 90, 3 } }

	,{ { ANCA_DIR | VERTI , "anca direita vertical"} , { 90, 3 } }
	,{ { ANCA_DIR | ABRIR , "anca direita abrir"} , { 90, 3 } }
	,{ { ANCA_DIR | AVANTE, "anca direita avante "} , { 90, 3 } }

	,{ { ANCA_ESQ | VERTI , "anca esquerda vertical"} , { 90, 3 } }
	,{ { ANCA_ESQ | ABRIR , "anca esquerda abrir"} , { 90, 3 } }
	,{ { ANCA_ESQ | AVANTE, "anca esquerda avante "} , { 90, 3 } }
/*
	,{ { TRONCO | LADO ,    "tronco lateral"} , { 90, 3 } }
	,{ { TRONCO | FRENTE ,  "tronco longitudinal"} , { 90, 3 } }

	,{ { BRA_DIR | LADO ,    "braco direito lado   "} , { 90, 3 } }
	,{ { BRA_DIR | FRENTE ,  "braco direito frente "} , { 90, 3 } }
*/

};

int ExchangeSerialData(int device, char *ord_msg, char *ret_msg);

/*==================================================*/


int main(void)
{
	int dev;
	int ret;
	char *d_name = DEVICENAME;
	char order_msg[3], pic_data[3];
	struct termio io_save;
	int n=0;


	dev = open(d_name, O_RDWR+O_NDELAY);
	if (dev<0) 
	{
	  printf("Cannot open device\n");
	  exit(2);
	}

	ioctl(dev, TCGETA, &io_save);

	if (ioctl(dev, TCSETA, &io_parm)<0) 
	{
	  printf("CAnnot change parameters\n");
	  exit(2);
	}
	ioctl(dev, TCSETA, &io_save);


/*Do your stuff here
============================================================= */

	do
	{
		order_msg[0] = MSG_REQ | HumanoidJointSequence[n%TOT_JOINTS].jointID.jointNum ;
		order_msg[1] =           HumanoidJointSequence[n%TOT_JOINTS].jointParams.jointPos ;
		order_msg[2] =           HumanoidJointSequence[n%TOT_JOINTS].jointParams.jointVel ;

		pic_data[0]=0;
		pic_data[1]=0;
		pic_data[2]=0;

		ret = ExchangeSerialData(dev, order_msg, pic_data);
		if(! ret)
		{
			printf("Correu mal!\n");
			exit(1);
		}

		printf("%s\t%d\t%d\t%d\n", HumanoidJointSequence[n%TOT_JOINTS].jointID.jointName,
								pic_data[0], 
								pic_data[1], 
								pic_data[2]);

	} while (n++ < 200);


/* ============================================================= */

	close(dev);
	exit(0);
}


/* ============================================================= */

int ExchangeSerialData(int device, char *ord_msg, char *ret_msg)
{
	int ret, i;
	char ch;

	/*
	ret_msg[0] = 1;
	ret_msg[1] = 3;
	ret_msg[2] = 5;
	return 1;
	*/

	if( ! ord_msg ) 
	{
		printf(".\n");
		return 0;   /*Cannot proceed: no message to send*/
	}

	ret = write( device, ord_msg, 3);
	if(ret < 3)
	{
		printf("Could not write all bytes: Only %d were written\n", ret);
		return 0;
	}

	if( ! ret_msg ) 	
	{
		printf("+\n");
		return 0;   /*No valid buffer for return values...*/ 
	}

	if( ! (ord_msg[0] & MSG_REQ ) )
	{
		printf("*\n");
		return 1;   /*it is not a request message. No need to proceed*/
	}


	/*OK. Now wait for data to be returned from PIC....*/
	do
	{
		ret=read(device, &ch,1);
		if(ret < 0)
		{
			printf("Could not read from device. Aborting.\n");
			close(device);
			exit(1);
		}
		if(ret) ord_msg[i++] = ch;
	} while(i <= 3);

	return 1;  /*Sucessful return*/

}
