
#ifndef _SERIAL_H_
#define _SERIAL_H_

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>

#include <qtimer.h>

#ifndef FALSE
#define FALSE	0
#endif

#ifndef TRUE
#define TRUE	1
#endif


// *** FLOW CONTROL ***
#define FLOW_UNDIFINED	-1
#define FLOW_OFF		0
#define FLOW_HARDWARE	1
#define FLOW_XONXOFF	2

// *** PARITY ***
#define PAR_UNDIFINED	-1
#define PAR_NONE		0
#define PAR_ODD			1
#define PAR_EVEN		2
#define PAR_SPACE		3

// *** DATA BITS ***
#define DATA_UNDIFINED	-1
#define DATA_5			0
#define DATA_6			1
#define DATA_7			2
#define DATA_8			3

// *** STOP BITS ***
#define STOP_UNDIFINED	-1
#define STOP_1			0 
#define STOP_2			1

// *** BAUD RATE ***
#define BAUD_UNDIFINED	-1
#define BAUD50			0
#define BAUD75			1
#define BAUD110			2
#define BAUD134			3
#define BAUD150			4
#define BAUD200			5
#define BAUD300			6
#define BAUD600			7
#define BAUD1200		8
#define BAUD1800		9
#define BAUD2400		10
#define BAUD4800		11
#define BAUD9600		12
#define BAUD19200		13
#define BAUD38400		14
#define BAUD57600		15
#define BAUD115200		16

#define TIMEOUT_UNDIFINED	-1


typedef struct _PortSettings {
    int BaudRate;
    int Parity;
    int DataBits;
    int StopBits;
    int FlowControl;
    long CharDelay;
    long Timeout;
	bool EnableTimeout;
} PortSettings;



class Serial
{
	int fd;
	struct termios oldtio, newtio;
	int last_error;
	bool is_open;
	char port_name[15];
	PortSettings port_settings;

public:
	Serial();
	//~Serial();

	bool Open();

	bool IsOpen();
	void Close();

	void GetErrorMsg(char *);

	void SetName(const char *);
	void Name(char *);

	bool SetBaudRate(int);
	int BaudRate();

	bool SetParity(int);
	int Parity();

	bool SetDataBits(int);
	int DataBits();

	bool SetStopBits(int);
	int StopBits();

	bool SetFlowControl(int);
	int FlowControl();

	void SetTimeout(long);
	long Timeout();

	void SetCharDelay(long);
	long CharDelay();

	void EnableTimeout(bool);
	bool isTimeout();

	int Read(unsigned char *buf, size_t count);
	int Write(const unsigned char *buf, size_t count);
};


#endif

