
#include "serial.h"

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <qstring.h>

#include <qmessagebox.h>
// *** PRIVATE FUNCTIONS ***


// *** PUBLIC FUNCTIONS ***

Serial::Serial()
{
	is_open = FALSE;
	last_error = 0;

	port_settings.FlowControl = FLOW_UNDIFINED;
	port_settings.Parity = PAR_UNDIFINED;
	port_settings.DataBits = DATA_UNDIFINED;
	port_settings.StopBits = STOP_UNDIFINED;
	port_settings.BaudRate = BAUD_UNDIFINED;
	port_settings.Timeout = TIMEOUT_UNDIFINED;
	port_settings.EnableTimeout = TRUE;

	strcpy(port_name, "");
}


// Serial::~Serial()
// {
// }


bool Serial::Open()
{
	QString aux;

	fd = (open(port_name, O_RDWR | O_NOCTTY));

	if (fd < 0) {
		last_error = errno;
		return FALSE;
	}

	tcgetattr(fd, &oldtio);				// save current port settings

    bzero(&newtio, sizeof(newtio));

	// *** C_CFLAG ***

	// Baud rate and Data bits
    newtio.c_cflag = port_settings.BaudRate | CLOCAL | CREAD;

	// Data bits
	switch (port_settings.DataBits) {
		case DATA_5:
			 newtio.c_cflag |= CS5; break;
		case DATA_6:
			 newtio.c_cflag |= CS6; break;
		case DATA_7:
			 newtio.c_cflag |= CS7; break;
		case DATA_8:
			 newtio.c_cflag |= CS8; break;
	}

	// Parity
	if ( (port_settings.Parity == PAR_EVEN) || (port_settings.Parity == PAR_ODD) )
		 newtio.c_cflag |= PARENB;

	if (port_settings.Parity == PAR_ODD)
		newtio.c_cflag |= PARODD;

	// Stop bits
	if (port_settings.StopBits == STOP_2)
		newtio.c_cflag |= CSTOPB;

	// Flow control (Hardware)
	if (port_settings.FlowControl == FLOW_HARDWARE) {
		newtio.c_cflag |= CRTSCTS;
	}

	// *** L_FLAG ***

    // set input mode (non-canonical, no echo,...) 
    newtio.c_lflag = 0;

	// *** I_FLAG ***

	// Parity related
	if ( (port_settings.Parity == PAR_EVEN) || (port_settings.Parity == PAR_ODD) )
    	newtio.c_iflag = INPCK;
    	//newtio.c_iflag = INPCK | ISTRIP;
		//newtio.c_iflag = IGNPAR;

	// Software (Xon/Xoff) control
	if (port_settings.FlowControl == FLOW_XONXOFF)
		newtio.c_iflag = IXON | IXOFF | IXANY;


	// *** O_FLAG ***
	
    newtio.c_oflag = 0;

	// TIME OUT
    newtio.c_cc[VTIME] = port_settings.CharDelay;
    newtio.c_cc[VMIN]  = 0;

	if (port_settings.EnableTimeout) {
		fcntl(fd, F_SETFL, 0);
	}
	else {
		fcntl(fd, F_SETFL, FNDELAY);
	}

    tcflush(fd, TCIFLUSH);

    tcsetattr(fd,TCSANOW,&newtio);

	is_open = TRUE;

	return TRUE;
}


void Serial::Close()
{
	tcsetattr(fd,TCSANOW,&oldtio);

	if (is_open)
		close(fd);
}


void Serial::GetErrorMsg(char *str)
{
	strcpy(str, strerror(last_error));		
}


bool Serial::IsOpen()
{
	return is_open;
}


void Serial::SetName(const char *name)
{
	strcpy(port_name, name);
}


void Serial::Name(char *name)
{
	strcpy(name, port_name);
}


bool Serial::SetBaudRate(int br)
{
	switch (br) {
		case BAUD50:		port_settings.BaudRate = B50;		return TRUE;
		case BAUD75:		port_settings.BaudRate = B75;		return TRUE;
		case BAUD110:		port_settings.BaudRate = B110;		return TRUE;
		case BAUD134:		port_settings.BaudRate = B134;		return TRUE;
		case BAUD150:		port_settings.BaudRate = B150;		return TRUE;
		case BAUD200:		port_settings.BaudRate = B200;		return TRUE;
		case BAUD300:		port_settings.BaudRate = B300;		return TRUE;
		case BAUD600:		port_settings.BaudRate = B600;		return TRUE;
		case BAUD1200:		port_settings.BaudRate = B1200;		return TRUE;
		case BAUD1800:		port_settings.BaudRate = B1800;		return TRUE;
		case BAUD2400:		port_settings.BaudRate = B2400;		return TRUE;
		case BAUD4800:		port_settings.BaudRate = B4800;		return TRUE;
		case BAUD9600:		port_settings.BaudRate = B9600;		return TRUE;
		case BAUD19200:		port_settings.BaudRate = B19200;	return TRUE;
		case BAUD38400:		port_settings.BaudRate = B38400;	return TRUE;
		case BAUD57600:		port_settings.BaudRate = B57600;	return TRUE;
		case BAUD115200:	port_settings.BaudRate = B115200;	return TRUE;
		default: return FALSE;
	}
}


int Serial::BaudRate(void)
{
	return port_settings.BaudRate;
}


bool Serial::SetParity(int par)
{
	switch (par) {
		case PAR_NONE:	port_settings.Parity = PAR_NONE;	return TRUE;
		case PAR_ODD:	port_settings.Parity = PAR_ODD;		return TRUE;
		case PAR_EVEN:	port_settings.Parity = PAR_EVEN;	return TRUE;
		case PAR_SPACE:	port_settings.Parity = PAR_SPACE;	return TRUE;
		default: return FALSE;
	}
}


int Serial::Parity()
{
	return port_settings.Parity;
}


bool Serial::SetDataBits(int db)
{
	switch (db) {
		case DATA_5:	port_settings.DataBits = DATA_5; return TRUE;
		case DATA_6:	port_settings.DataBits = DATA_6; return TRUE;
		case DATA_7:	port_settings.DataBits = DATA_7; return TRUE;
		case DATA_8:	port_settings.DataBits = DATA_8; return TRUE;
		default: return FALSE;
	}
}


int Serial::DataBits()
{
	return port_settings.DataBits;
}


bool Serial::SetStopBits(int db)
{
	switch (db) {
		case STOP_1:	port_settings.StopBits = STOP_1; return TRUE;
		case STOP_2:	port_settings.StopBits = STOP_2; return TRUE;
		default: return FALSE;
	}
}


int Serial::StopBits()
{
	return port_settings.StopBits;
}


bool Serial::SetFlowControl(int fc)
{
	switch (fc) {
		case FLOW_OFF:
		case FLOW_HARDWARE:
		case FLOW_XONXOFF:
			port_settings.FlowControl = fc ;
			return TRUE;
		default:
			return FALSE;
	}
}


int Serial::FlowControl()
{
	return port_settings.FlowControl;
}


void Serial::SetTimeout(long to)
{
	port_settings.Timeout = to;
}


long Serial::Timeout()
{
	return port_settings.Timeout;
}


void Serial::SetCharDelay(long cd)
{
	port_settings.CharDelay = cd;
}


long Serial::CharDelay()
{
	return port_settings.CharDelay;
}



void Serial::EnableTimeout(bool eto)
{
	port_settings.EnableTimeout = eto;
}


bool Serial::isTimeout()
{
	return port_settings.EnableTimeout;
	char str[100];
	//QMessageBox::information( this,"Entrou no serial-write", str);
}


int Serial::Read(unsigned char *buf, size_t count)
{
	unsigned int b;
	time_t itime, atime;

	b = 0;

	time(&itime);
	time(&atime);

	do{
	//while ( (b < count) && (difftime(atime, itime) < port_settings.Timeout) ) {
		b += read(fd, buf+b, count-b);
		time(&atime);
	} while ( (b < count) && (difftime(atime, itime) < port_settings.Timeout)) ;
	return b;
}


int Serial::Write(const unsigned char *buf, size_t count)
{
	return write(fd, buf, count);
}

